#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of python-max17040 library.
#
# python-max17040 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-max17040 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with python-max17040.  If not, see <http://www.gnu.org/licenses/>

# 2018-10-15 David Goncalves <david@lestat.st>
#            Python class to communicate with a MAX17040 chip on I2C bus.
#            It's a 1-Cell Fuel Gauge with ModelGauge from Maxim Integrated.

import smbus
import sys


class max17040:

    # Internal register addresses
    REG_VCELL = 0x2
    REG_SOC = 0x4
    REG_MODE = 0x6
    REG_VERSION = 0x8
    REG_RCOMP = 0xc
    REG_COMMAND = 0xfe

    # Values for mode register
    MODE_QSTRT = 0x4000       # Quick Start

    # Values for command register
    CMD_POR = 0x54            # Power-On Reset

    def __init__(self, address=0x36, bus=1, debug=False):
        self._debug = debug
        self.i2c_addr = address
        self.i2c_bus = smbus.SMBus(bus)

        self._debug_message("Accessing MAX17040 at address 0x%x on bus %s" % (address, bus))

    def _debug_message(self, message="", error=False):
        """
        Prints debug messages to console when debuging mode is activated
        """

        if (self._debug is True) or (error is True):
            print("%s %s" % (("[DEBUG]" if error is False else "[ERROR]"), message))
            sys.stdout.flush()
            if (error is True):
                raise Exception(message)

    def _write_data(self, address, data):
        """
        Private method to write data to the chip
        """

        self._debug_message("> Writing %s bytes to register 0x%x [%s]" % (len(data), address, ' '.join(["0x%02X" % x for x in data])))
        self.i2c_bus.write_i2c_block_data(self.i2c_addr, address, data)

    def _read_data(self, address, size):
        """
        Private method to read data from the chip
        """

        self._debug_message("> Reading %s bytes from register 0x%x" % (size, address))
        data = self.i2c_bus.read_i2c_block_data(self.i2c_addr, address, size)
        self._debug_message("> MAX17040 response : [%s]" % ' '.join(["0x%02X" % x for x in data]))
        return(data)

    def GetFirmwareVersion(self):
        """
        Returns the chip's firmware version
        """

        self._debug_message("Reading firmware version")
        data = self._read_data(self.REG_VERSION, 2)
        version = (data[0] << 8) + data[1]
        self._debug_message("MAX17040 version : %s" % version)
        return(version)

    def GetCellVoltage(self):
        """
        Returns the cell voltage in millivolts
        """

        self._debug_message("Reading cell voltage")
        data = self._read_data(self.REG_VCELL, 2)
        voltage = (((data[0] << 8) + data[1]) >> 4) * 1.25
        self._debug_message("Cell voltage is %smV" % voltage)
        return(int(voltage))

    def GetStateOfCharge(self):
        """
        Get the SOC by reading the appropriate register.

        This method uses the high precision value from the
        register and round it to two decimals.
        """

        self._debug_message("Reading State Of Charge")
        data = self._read_data(self.REG_SOC, 2)
        soc = round(data[0] + (data[1] * (1/256.0)), 2)
        self._debug_message("SOC = %s%%" % soc)
        return(100.0 if soc > 100 else soc)
